library(glmnet)
library(pROC)
library(DescTools)
library(car)
library(dplyr)
library(pscl) # McFaddens R squared
library(writexl) # Lage Excel-fil
library(readr)

# Laster inn datasettet
df <- read_delim("C:/Users/marti/OneDrive - NTNU/Masteroppgave/data/data.csv", 
                 delim = ";", escape_double = FALSE, trim_ws = TRUE)

# Rydder datasettet, fjerner kolonner vi ikke har bruk for
df = df[,-3:-40]
df = df[,-1]

# R takler dårlig variabelnavn betående av tall
# Setter derfor en X foran hver variabelnavn
colnames(df) = paste('X', colnames(df), sep = '') 
colnames(df)[colnames(df) == "Xregnaar"] ="regnaar"
colnames(df)[colnames(df) == "Xbankrupt_fs"] ="bankrupt_fs"

# Viser datasettet
head(df)
# Viser kolonnenavnene
colnames(df)

# Variablene som skal brukes senere
variabler = c("X1","X2", "X3", "X4", "X5", "X6", "X7", "X8", "X9", "X10",
             "X11", "X12", "X13", "X14", "X15", "X16", "X17", "X18", "X19", "X20",
             "X21", "X22", "X23", "X24", "X25", "X26", "X27", "X28", "X29", "X30",
             "X31", "X32", "X33", "X34", "X35", "X36", "X37", "X38", "X39", "X40",
             "X41", "X42", "X43", "X44", "X45", "X46", "X47", "X48", "X49", "X50",
             "X51", "X52", "X53", "X54", "X55", "X56", "X57", "X58", "X59", "X60",
             "X61", "X62", "X63", "X64", "X65", "X66", "X67", "X68", "X69", "X70",
             "X71", "X72", "X73", "X74", "X75", "X76", "X77", "X78", "X79", "X80",
             "X81", "X82", "X83", "X84", "X85", "X86", "X87", "X88", "X89", "X90",
             "X91", "X92", "X93", "X94", "X95", "X96", "X97", "X98", "X99", "X100",
             "X101", "X102", "X103", "X104", "X105", "X106", "X107", "X108", "X109",
             "X110", "X111", "X112", "X113", "X114", "X115", "X116", "X117", "X118",
             "X119", "X120", "X121", "X122", "X123", "X124", "X125", "X126", "X127",
             "X128", "X129", "X130", "X131", "X132", "X133", "X134", "X135", "X136",
             "X137", "X138", "X139", "X140", "X141", "X142", "X143", "X144", "X145",
             "X146", "X147", "X148", "X149", "X150", "X151", "X152", "X153", "X154",
             "X155")

#Fold 1

testaar_1 = 2010

# Definerer data som skal brukes
df_train_1  = as.matrix(df[(df$regnaar<testaar_1),c(variabler,'bankrupt_fs')])
df_test_1   = as.matrix(df[(df$regnaar==testaar_1),c(variabler,'bankrupt_fs')])
X_train_1 = df_train_1[,variabler]
X_test_1  = df_test_1[,variabler]
y_train_1 = df_train_1[,'bankrupt_fs']
y_test_1  = df_test_1[,'bankrupt_fs']


# Tuner lambdaverdi (TAR CIRKA 15 MIN)
set.seed(1)
cvfit_train_1 = cv.glmnet(X_train_1, y_train_1, alpha=1, family = "binomial", nfolds = 3, type.measure = "auc",standardize = TRUE)

# Trener LASSO med den tunede lambdaverdien
glm.model_1 = glmnet(X_train_1, y_train_1, alpha=1, family = "binomial", lambda = cvfit_train_1$lambda.1se, standardize = TRUE)

# Trekker ut de utvalgte variablene
variables_selected_1 = as.matrix(coef(glm.model_1))
variables_selected_1 = rownames(variables_selected_1)[variables_selected_1!=0]
variables_selected_1 = variables_selected_1[!variables_selected_1 == '(Intercept)']


# LASSO over et rutenett
grid_1 = cvfit_train_1$lambda.1se*10^seq(6,0, length = 200)
lasso_path_1 = glmnet(X_train_1[,variables_selected_1], y_train_1, alpha = 1, lambda = grid_1, family = 'binomial',standardize = TRUE)

# Lager 2 stk plot
# AUC og lasso path
plot(lasso_path_1, "lambda", xlim=c(log(cvfit_train_1$lambda.1se),-3.8), col=1:15,ylim=c(-1,2), ylab="Standardiserte koeffisienter", lwd=2)
plot(cvfit_train_1, xlim=c(log(cvfit_train_1$lambda.1se),-3.8), xlab="")

rm(cvfit_train_1, df_test_1, df_train_1, glm.model_1)
rm(lasso_path_1, X_test_1, X_train_1, grid_1)
rm(testaar_1, variables_selected_1, y_test_1, y_train_1)

#Fold 2

testaar_2 = 2011

# Definerer data som skal brukes
df_train_2  = as.matrix(df[(df$regnaar<testaar_2 & df$regnaar>(testaar_2-5)),c(variabler,'bankrupt_fs')])
df_test_2   = as.matrix(df[(df$regnaar==testaar_2),c(variabler,'bankrupt_fs')])
X_train_2 = df_train_2[,variabler]
X_test_2  = df_test_2[,variabler]
y_train_2 = df_train_2[,'bankrupt_fs']
y_test_2  = df_test_2[,'bankrupt_fs']


# Tuner lambdaverdi (TAR CIRKA 15 MIN)
set.seed(1)
cvfit_train_2 = cv.glmnet(X_train_2, y_train_2, alpha=1, family = "binomial", nfolds = 3, type.measure = "auc",standardize = TRUE)

# Trener LASSO med den tunede lambdaverdien
glm.model_2 = glmnet(X_train_2, y_train_2, alpha=1, family = "binomial", lambda = cvfit_train_2$lambda.1se, standardize = TRUE)


# Trekker ut de utvalgte variablene
variables_selected_2 = as.matrix(coef(glm.model_2))
variables_selected_2 = rownames(variables_selected_2)[variables_selected_2!=0]
variables_selected_2 = variables_selected_2[!variables_selected_2 == '(Intercept)']

# LASSO over et rutenett
grid_2 = cvfit_train_2$lambda.1se*10^seq(6,0, length = 200)
lasso_path_2 = glmnet(X_train_2[,variables_selected_2], y_train_2, alpha = 1, lambda = grid_2, family = 'binomial',standardize = TRUE)

# Lager 2 stk plot
# AUC og lasso path
plot(lasso_path_2, "lambda", xlim=c(log(cvfit_train_2$lambda.1se),-3.8), col=1:20,ylim=c(-1,2), ylab="Standardiserte koeffisienter", lwd=2)
plot(cvfit_train_2, xlim=c(log(cvfit_train_2$lambda.1se),-3.8), xlab="")

rm(cvfit_train_2, df_test_2, df_train_2, glm.model_2)
rm(lasso_path_2, X_test_2, X_train_2, grid_2)
rm(testaar_2, variables_selected_2, y_test_2, y_train_2)



#Fold 3

testaar_3 = 2012

# Definerer data som skal brukes
df_train_3  = as.matrix(df[(df$regnaar<testaar_3 & df$regnaar>(testaar_3-5)),c(variabler,'bankrupt_fs')])
df_test_3   = as.matrix(df[(df$regnaar==testaar_3),c(variabler,'bankrupt_fs')])
X_train_3 = df_train_3[,variabler]
X_test_3  = df_test_3[,variabler]
y_train_3 = df_train_3[,'bankrupt_fs']
y_test_3  = df_test_3[,'bankrupt_fs']


# Tuner lambdaverdi (TAR CIRKA 15 MIN)
set.seed(1)
cvfit_train_3 = cv.glmnet(X_train_3, y_train_3, alpha=1, family = "binomial", nfolds = 3, type.measure = "auc",standardize = TRUE)

# Trener LASSO med den tunede lambdaverdien
glm.model_3 = glmnet(X_train_3, y_train_3, alpha=1, family = "binomial", lambda = cvfit_train_3$lambda.1se, standardize = TRUE)

# Trekker ut de utvalgte variablene
variables_selected_3 = as.matrix(coef(glm.model_3))
variables_selected_3 = rownames(variables_selected_3)[variables_selected_3!=0]
variables_selected_3 = variables_selected_3[!variables_selected_3 == '(Intercept)']

# LASSO over et rutenett
grid_3 = cvfit_train_3$lambda.1se*10^seq(6,0, length = 200)
lasso_path_3 = glmnet(X_train_3[,variables_selected_3], y_train_3, alpha = 1, lambda = grid_3, family = 'binomial',standardize = TRUE)

# Lager 2 stk plot
# AUC og lasso path
plot(lasso_path_3, "lambda", xlim=c(log(cvfit_train_3$lambda.1se),-3.8), col=1:25,ylim=c(-1,2), ylab="Standardiserte koeffisienter", lwd=2)
plot(cvfit_train_3, xlim=c(log(cvfit_train_3$lambda.1se),-3.8), xlab="")

rm(cvfit_train_3, df_test_3, df_train_3, glm.model_3)
rm(lasso_path_3, X_test_3, X_train_3, grid_3)
rm(testaar_3, variables_selected_3, y_test_3, y_train_3)

#Fold 4

testaar_4 = 2013

# Definerer data som skal brukes
df_train_4  = as.matrix(df[(df$regnaar<testaar_4 & df$regnaar>(testaar_4-5)),c(variabler,'bankrupt_fs')])
df_test_4   = as.matrix(df[(df$regnaar==testaar_4),c(variabler,'bankrupt_fs')])
X_train_4 = df_train_4[,variabler]
X_test_4  = df_test_4[,variabler]
y_train_4 = df_train_4[,'bankrupt_fs']
y_test_4  = df_test_4[,'bankrupt_fs']


# Tuner lambdaverdi (TAR CIRKA 15 MIN)
set.seed(1)
cvfit_train_4 = cv.glmnet(X_train_4, y_train_4, alpha=1, family = "binomial", nfolds = 3, type.measure = "auc",standardize = TRUE)

# Trener LASSO med den tunede lambdaverdien
glm.model_4 = glmnet(X_train_4, y_train_4, alpha=1, family = "binomial", lambda = cvfit_train_4$lambda.1se, standardize = TRUE)

# Trekker ut de utvalgte variablene
variables_selected_4 = as.matrix(coef(glm.model_4))
variables_selected_4 = rownames(variables_selected_4)[variables_selected_4!=0]
variables_selected_4 = variables_selected_4[!variables_selected_4 == '(Intercept)']

# LASSO over et rutenett
grid_4 = cvfit_train_4$lambda.1se*10^seq(6,0, length = 200)
lasso_path_4 = glmnet(X_train_4[,variables_selected_4], y_train_4, alpha = 1, lambda = grid_4, family = 'binomial',standardize = TRUE)

# Lager 2 stk plot
# AUC og lasso path
plot(lasso_path_4, "lambda", xlim=c(log(cvfit_train_4$lambda.1se),-3.8), col=1:15,ylim=c(-1,2), ylab="Standardiserte koeffisienter", lwd=2)
plot(cvfit_train_4, xlim=c(log(cvfit_train_4$lambda.1se),-3.8), xlab="")

rm(cvfit_train_4, df_test_4, df_train_4, glm.model_4)
rm(lasso_path_4, X_test_4, X_train_4, grid_4)
rm(testaar_4, variables_selected_4, y_test_4, y_train_4)


#Fold 5

testaar_5 = 2014

# Definerer data som skal brukes
df_train_5  = as.matrix(df[(df$regnaar<testaar_5 & df$regnaar>(testaar_5-5)),c(variabler,'bankrupt_fs')])
df_test_5   = as.matrix(df[(df$regnaar==testaar_5),c(variabler,'bankrupt_fs')])
X_train_5 = df_train_5[,variabler]
X_test_5  = df_test_5[,variabler]
y_train_5 = df_train_5[,'bankrupt_fs']
y_test_5  = df_test_5[,'bankrupt_fs']


# Tuner lambdaverdi (TAR CIRKA 15 MIN)
# Hadde her problemer med at tuningen aldri treffer en lambdaverdi. Setter derfor lambda til to gitte verdier
set.seed(1)
cvfit_train_5 = cv.glmnet(X_train_5, y_train_5, lambda = c(0.00073,0.0008),alpha=1, family = "binomial", nfolds = 3, type.measure = "auc",standardize = TRUE)

# Trener LASSO med den tunede lambdaverdien
glm.model_5 = glmnet(X_train_5, y_train_5, alpha=1, family = "binomial", lambda = cvfit_train_5$lambda.min, standardize = TRUE)

# Trekker ut de utvalgte variablene
variables_selected_5 = as.matrix(coef(glm.model_5))
variables_selected_5 = rownames(variables_selected_5)[variables_selected_5!=0]
variables_selected_5 = variables_selected_5[!variables_selected_5 == '(Intercept)']

# LASSO over et rutenett
grid_5 = cvfit_train_5$lambda.min*10^seq(6,0, length = 200)
lasso_path_5 = glmnet(X_train_5[,variables_selected_5], y_train_5, alpha = 1, lambda = grid_5, family = 'binomial',standardize = TRUE)

# Lager 2 stk plot
# AUC og lasso path
plot(lasso_path_5, "lambda", xlim=c(log(cvfit_train_5$lambda.min),-3.8), col=1:15,ylim=c(-1,2), ylab="Standardiserte koeffisienter", lwd=2)
# Får ikke laget dette plottet pga problemet som nevnt tidligere.
#plot(cvfit_train_5, xlim=c(log(cvfit_train_5$lambda.min),-3.8), xlab="")

rm(cvfit_train_5, df_test_5, df_train_5, glm.model_5)
rm(lasso_path_5, X_test_5, X_train_5, grid_5)
rm(testaar_5, variables_selected_5, y_test_5, y_train_5)





#Fold 6

testaar_6 = 2015

# Definerer data som skal brukes
df_train_6  = as.matrix(df[(df$regnaar<testaar_6 & df$regnaar>(testaar_6-5)),c(variabler,'bankrupt_fs')])
df_test_6   = as.matrix(df[(df$regnaar==testaar_6),c(variabler,'bankrupt_fs')])
X_train_6 = df_train_6[,variabler]
X_test_6  = df_test_6[,variabler]
y_train_6 = df_train_6[,'bankrupt_fs']
y_test_6  = df_test_6[,'bankrupt_fs']


# Tuner lambdaverdi (TAR CIRKA 15 MIN)
set.seed(1)
cvfit_train_6 = cv.glmnet(X_train_6, y_train_6, alpha=1, family = "binomial", nfolds = 3, type.measure = "auc",standardize = TRUE)

# Trener LASSO med den tunede lambdaverdien
glm.model_6 = glmnet(X_train_6, y_train_6, alpha=1, family = "binomial", lambda = cvfit_train_6$lambda.1se, standardize = TRUE)

# Trekker ut de utvalgte variablene
variables_selected_6 = as.matrix(coef(glm.model_6))
variables_selected_6 = rownames(variables_selected_6)[variables_selected_6!=0]
variables_selected_6 = variables_selected_6[!variables_selected_6 == '(Intercept)']

# LASSO over et rutenett
grid_6 = cvfit_train_6$lambda.1se*10^seq(6,0, length = 200)
lasso_path_6 = glmnet(X_train_6[,variables_selected_6], y_train_6, alpha = 1, lambda = grid_6, family = 'binomial',standardize = TRUE)

# Lager 2 stk plot
# AUC og lasso path
plot(lasso_path_6, "lambda", xlim=c(log(cvfit_train_6$lambda.1se),-3.8), col=1:15,ylim=c(-1,2), ylab="Standardiserte koeffisienter", lwd=2)
plot(cvfit_train_6, xlim=c(log(cvfit_train_6$lambda.1se),-3.8), xlab="")

rm(cvfit_train_6, df_test_6, df_train_6, glm.model_6)
rm(lasso_path_6, X_test_6, X_train_6, grid_6)
rm(testaar_6, variables_selected_6, y_test_6, y_train_6)




#Fold 7

testaar_7 = 2016

# Definerer data som skal brukes
df_train_7  = as.matrix(df[(df$regnaar<testaar_7 & df$regnaar>(testaar_7-5)),c(variabler,'bankrupt_fs')])
df_test_7   = as.matrix(df[(df$regnaar==testaar_7),c(variabler,'bankrupt_fs')])
X_train_7 = df_train_7[,variabler]
X_test_7  = df_test_7[,variabler]
y_train_7 = df_train_7[,'bankrupt_fs']
y_test_7  = df_test_7[,'bankrupt_fs']


# Tuner lambdaverdi (TAR CIRKA 15 MIN)
set.seed(1)
cvfit_train_7 = cv.glmnet(X_train_7, y_train_7, alpha=1, family = "binomial", nfolds = 3, type.measure = "auc",standardize = TRUE)

# Trener LASSO med den tunede lambdaverdien
glm.model_7 = glmnet(X_train_7, y_train_7, alpha=1, family = "binomial", lambda = cvfit_train_7$lambda.1se, standardize = TRUE)

# Trekker ut de utvalgte variablene
variables_selected_7 = as.matrix(coef(glm.model_7))
variables_selected_7 = rownames(variables_selected_7)[variables_selected_7!=0]
variables_selected_7 = variables_selected_7[!variables_selected_7 == '(Intercept)']

# LASSO over et rutenett
grid_7 = cvfit_train_7$lambda.1se*10^seq(6,0, length = 200)
lasso_path_7 = glmnet(X_train_7[,variables_selected_7], y_train_7, alpha = 1, lambda = grid_7, family = 'binomial',standardize = TRUE)

# Lager 2 stk plot
# AUC og lasso path
plot(lasso_path_7, "lambda", xlim=c(log(cvfit_train_7$lambda.1se),-3.8), col=1:20,ylim=c(-1,2), ylab="Standardiserte koeffisienter", lwd=2)
plot(cvfit_train_7, xlim=c(log(cvfit_train_7$lambda.1se),-3.8), xlab="")

rm(cvfit_train_7, df_test_7, df_train_7, glm.model_7)
rm(lasso_path_7, X_test_7, X_train_7, grid_7)
rm(testaar_7, variables_selected_7, y_test_7, y_train_7)



#Fold 8

testaar_8 = 2017

# Definerer data som skal brukes
df_train_8  = as.matrix(df[(df$regnaar<testaar_8 & df$regnaar>(testaar_8-5)),c(variabler,'bankrupt_fs')])
df_test_8   = as.matrix(df[(df$regnaar==testaar_8),c(variabler,'bankrupt_fs')])
X_train_8 = df_train_8[,variabler]
X_test_8  = df_test_8[,variabler]
y_train_8 = df_train_8[,'bankrupt_fs']
y_test_8  = df_test_8[,'bankrupt_fs']


# Tuner lambdaverdi (TAR CIRKA 15 MIN)
set.seed(1)
cvfit_train_8 = cv.glmnet(X_train_8, y_train_8, alpha=1, family = "binomial", nfolds = 3, type.measure = "auc",standardize = TRUE)

# Trener LASSO med den tunede lambdaverdien
glm.model_8 = glmnet(X_train_8, y_train_8, alpha=1, family = "binomial", lambda = cvfit_train_8$lambda.1se, standardize = TRUE)

# Trekker ut de utvalgte variablene
variables_selected_8 = as.matrix(coef(glm.model_8))
variables_selected_8 = rownames(variables_selected_8)[variables_selected_8!=0]
variables_selected_8 = variables_selected_8[!variables_selected_8 == '(Intercept)']

# LASSO over et rutenett
grid_8 = cvfit_train_8$lambda.1se*10^seq(6,0, length = 200)
lasso_path_8 = glmnet(X_train_8[,variables_selected_8], y_train_8, alpha = 1, lambda = grid_8, family = 'binomial',standardize = TRUE)

# Lager 2 stk plot
# AUC og lasso path
plot(lasso_path_8, "lambda", xlim=c(log(cvfit_train_8$lambda.1se),-3.8), col=1:20,ylim=c(-1,2), ylab="Standardiserte koeffisienter", lwd=2)
plot(cvfit_train_8, xlim=c(log(cvfit_train_8$lambda.1se),-3.8), xlab="")

rm(cvfit_train_8, df_test_8, df_train_8, glm.model_8)
rm(lasso_path_8, X_test_8, X_train_8, grid_8)
rm(testaar_8, variables_selected_8, y_test_8, y_train_8)




#Fold 9

testaar_9 = 2018

# Definerer data som skal brukes
df_train_9  = as.matrix(df[(df$regnaar<testaar_9 & df$regnaar>(testaar_9-5)),c(variabler,'bankrupt_fs')])
df_test_9   = as.matrix(df[(df$regnaar==testaar_9),c(variabler,'bankrupt_fs')])
X_train_9 = df_train_9[,variabler]
X_test_9  = df_test_9[,variabler]
y_train_9 = df_train_9[,'bankrupt_fs']
y_test_9  = df_test_9[,'bankrupt_fs']


# Tuner lambdaverdi (TAR CIRKA 15 MIN)
set.seed(1)
cvfit_train_9 = cv.glmnet(X_train_9, y_train_9, alpha=1, family = "binomial", nfolds = 3, type.measure = "auc",standardize = TRUE)

# Trener LASSO med den tunede lambdaverdien
glm.model_9 = glmnet(X_train_9, y_train_9, alpha=1, family = "binomial", lambda = cvfit_train_9$lambda.1se, standardize = TRUE)

# Trekker ut de utvalgte variablene
variables_selected_9 = as.matrix(coef(glm.model_9))
variables_selected_9 = rownames(variables_selected_9)[variables_selected_9!=0]
variables_selected_9 = variables_selected_9[!variables_selected_9 == '(Intercept)']

# LASSO over et rutenett
grid_9 = cvfit_train_9$lambda.1se*10^seq(6,0, length = 200)
lasso_path_9 = glmnet(X_train_9[,variables_selected_9], y_train_9, alpha = 1, lambda = grid_9, family = 'binomial',standardize = TRUE)

# Lager 2 stk plot
# AUC og lasso path
plot(lasso_path_9, "lambda", xlim=c(log(cvfit_train_9$lambda.1se),-3.8), col=1:25,ylim=c(-1,2), ylab="Standardiserte koeffisienter", lwd=2)
plot(cvfit_train_9, xlim=c(log(cvfit_train_9$lambda.1se),-3.8), xlab="")

rm(cvfit_train_9, df_test_9, df_train_9, glm.model_9)
rm(lasso_path_9, X_test_9, X_train_9, grid_9)
rm(testaar_9, variables_selected_9, y_test_9, y_train_9)




#Fold 10

testaar_10 = 2019

# Definerer data som skal brukes
df_train_10  = as.matrix(df[(df$regnaar<testaar_10 & df$regnaar>(testaar_10-5)),c(variabler,'bankrupt_fs')])
df_test_10   = as.matrix(df[(df$regnaar==testaar_10),c(variabler,'bankrupt_fs')])
X_train_10 = df_train_10[,variabler]
X_test_10  = df_test_10[,variabler]
y_train_10 = df_train_10[,'bankrupt_fs']
y_test_10  = df_test_10[,'bankrupt_fs']


# Tuner lambdaverdi (TAR CIRKA 15 MIN)
set.seed(1)
cvfit_train_10 = cv.glmnet(X_train_10, y_train_10, alpha=1, family = "binomial", nfolds = 3, type.measure = "auc",standardize = TRUE)

# Trener LASSO med den tunede lambdaverdien
glm.model_10 = glmnet(X_train_10, y_train_10, alpha=1, family = "binomial", lambda = cvfit_train_10$lambda.1se, standardize = TRUE)

# Trekker ut de utvalgte variablene
variables_selected_10 = as.matrix(coef(glm.model_10))
variables_selected_10 = rownames(variables_selected_10)[variables_selected_10!=0]
variables_selected_10 = variables_selected_10[!variables_selected_10 == '(Intercept)']

# LASSO over et rutenett
grid_10 = cvfit_train_10$lambda.1se*10^seq(6,0, length = 200)
lasso_path_10 = glmnet(X_train_10[,variables_selected_10], y_train_10, alpha = 1, lambda = grid_10, family = 'binomial',standardize = TRUE)

# Lager 2 stk plot
# AUC og lasso path
plot(lasso_path_10, "lambda", xlim=c(log(cvfit_train_10$lambda.1se),-3.8), col=1:25,ylim=c(-1,4), ylab="Standardiserte koeffisienter", lwd=2)
plot(cvfit_train_10, xlim=c(log(cvfit_train_10$lambda.1se),-3.8), xlab="")

rm(cvfit_train_10, df_test_10, df_train_10, glm.model_10)
rm(lasso_path_10, X_test_10, X_train_10, grid_10)
rm(testaar_10, variables_selected_10, y_test_10, y_train_10)
